@@ -180,6 +180,19 @@ class BoxAnnotation(Annotation):  # pylint: disable=R0902
             EMBEDDING_VECTOR_KEY: self.embedding_vector,
         }
 
+    def __eq__(self, other):
+        return (
+            self.label == other.label
+            and self.x == other.x
+            and self.y == other.y
+            and self.width == other.width
+            and self.height == other.height
+            and self.reference_id == other.reference_id
+            and self.annotation_id == other.annotation_id
+            and sorted(self.metadata.items()) == sorted(other.metadata.items())
+            and self.embedding_vector == other.embedding_vector
+        )
+
 
 @dataclass
 class Point:
@@ -426,7 +439,7 @@ class KeypointsAnnotation(Annotation):
             label="face",
             keypoints=[Keypoint(100, 100), Keypoint(120, 120), Keypoint(visible=False), Keypoint(0, 0)],
             names=["point1", "point2", "point3", "point4"],
-            skeleton=[[0, 1], [1, 2], [1, 3], [2, 4]],
+            skeleton=[[0, 1], [1, 2], [1, 3], [2, 3]],
             reference_id="image_2",
             annotation_id="image_2_face_keypoints_1",
             metadata={"face_direction": "forward"},
@@ -464,11 +477,26 @@ class KeypointsAnnotation(Annotation):
             raise ValueError(
                 "The list of keypoints must be the same length as the list of names"
             )
+        if len(set(self.names)) != len(self.names):
+            seen = set()
+            for name in self.names:
+                if name in seen:
+                    raise ValueError(
+                        f"The keypoint name '{name}' is repeated in the list of names"
+                    )
+                seen.add(name)
+
+        max_segment_index = len(self.keypoints) - 1
         for segment in self.skeleton:
             if len(segment) != 2:
                 raise ValueError(
                     "The keypoints skeleton must contain a list of line segments with exactly 2 indices"
                 )
+            for index in segment:
+                if index > max_segment_index:
+                    raise ValueError(
+                        f"The skeleton index {index} is not a valid keypoint index"
+                    )
 
     @classmethod
     def from_json(cls, payload: dict):
@@ -911,6 +939,9 @@ class AnnotationList:
                 ), f"Unexpected annotation type: {type(annotation)}"
                 self.segmentation_annotations.append(annotation)
 
+    def items(self):
+        return self.__dict__.items()
+
     def __len__(self):
         return (
             len(self.box_annotations)
