@@ -27,6 +27,7 @@ ANNOTATION_UPDATE_KEY = "update"
 AUTOTAGS_KEY = "autotags"
 AUTOTAG_SCORE_THRESHOLD = "score_threshold"
 EXPORTED_ROWS = "exportedRows"
+EXPORTED_SCALE_TASK_INFO_ROWS = "exportedScaleTaskInfoRows"
 CAMERA_MODEL_KEY = "camera_model"
 CAMERA_PARAMS_KEY = "camera_params"
 CLASS_PDF_KEY = "class_pdf"
@@ -81,13 +82,14 @@ JOB_CREATION_TIME_KEY = "job_creation_time"
 KEYPOINTS_KEY = "keypoints"
 KEYPOINTS_NAMES_KEY = "names"
 KEYPOINTS_SKELETON_KEY = "skeleton"
-LAST_PAGE = "lastPage"
 LABEL_KEY = "label"
 LABELS_KEY = "labels"
 MASK_URL_KEY = "mask_url"
 MAX_PAYLOAD_SIZE = 0x1FFFFFE8  # Set to max string size since we currently convert payloads to strings for processing on the server-side
 MESSAGE_KEY = "message"
 METADATA_KEY = "metadata"
+MODEL_BUNDLE_NAME_KEY = "bundle_name"
+MODEL_TAGS_KEY = "tags"
 MODEL_ID_KEY = "model_id"
 MODEL_RUN_ID_KEY = "model_run_id"
 NAME_KEY = "name"
@@ -95,8 +97,9 @@ NEW_ITEMS = "new_items"
 NUCLEUS_ENDPOINT = "https://api.scale.com/v1/nucleus"
 NUM_SENSORS_KEY = "num_sensors"
 ORIGINAL_IMAGE_URL_KEY = "original_image_url"
-PAGE_SIZE = "pageSize"
-PAGE_TOKEN = "pageToken"
+PAGE_SIZE_KEY = "pageSize"
+PAGE_TOKEN_KEY = "pageToken"
+NEXT_TOKEN_KEY = "nextPageToken"
 P1_KEY = "p1"
 P2_KEY = "p2"
 POINTCLOUD_KEY = "pointcloud"
@@ -104,11 +107,14 @@ POINTCLOUD_LOCATION_KEY = "pointcloud_location"
 POINTCLOUD_URL_KEY = "pointcloud_url"
 POSITION_KEY = "position"
 PREDICTIONS_IGNORED_KEY = "predictions_ignored"
+PREDICTIONS_KEY = "predictions"
 PREDICTIONS_PROCESSED_KEY = "predictions_processed"
 REFERENCE_IDS_KEY = "reference_ids"
 REFERENCE_ID_KEY = "reference_id"
 BACKEND_REFERENCE_ID_KEY = "ref_id"  # TODO(355762): Our backend returns this instead of the "proper" key sometimes.
 REQUEST_ID_KEY = "requestId"
+SCALE_TASK_INFO_KEY = "scale_task_info"
+SCENE_KEY = "scene"
 SCENES_KEY = "scenes"
 SERIALIZED_REQUEST_KEY = "serialized_request"
 SEGMENTATIONS_KEY = "segmentations"
@@ -123,8 +129,9 @@ UPDATE_KEY = "update"
 UPLOAD_TO_SCALE_KEY = "upload_to_scale"
 URL_KEY = "url"
 VERTICES_KEY = "vertices"
+VIDEO_LOCATION_KEY = "video_location"
+VIDEO_URL_KEY = "video_url"
 VISIBLE_KEY = "visible"
-VIDEO_UPLOAD_TYPE_KEY = "video_upload_type"
 WIDTH_KEY = "width"
 YAW_KEY = "yaw"
 W_KEY = "w"
