@@ -5,28 +5,10 @@ import numpy as np
 
 try:
     from shapely.geometry import Polygon
-except ModuleNotFoundError:
-    import sys
+except (ModuleNotFoundError, OSError):
+    from ..package_not_installed import PackageNotInstalled
 
-    class Polygon:  # type: ignore
-        def __init__(self, *args, **kwargs):
-            """Object to make sure we only raise errors if actually trying to use shapely"""
-            if sys.platform.startswith("darwin"):
-                platform_specific_msg = (
-                    "Depending on Python environment used GEOS might need to be installed via "
-                    "`brew install geos`."
-                )
-            elif sys.platform.startswith("linux"):
-                platform_specific_msg = (
-                    "Depending on Python environment used GEOS might need to be installed via "
-                    "system package `libgeos-dev`."
-                )
-            else:
-                platform_specific_msg = "GEOS package will need to be installed see (https://trac.osgeo.org/geos/)"
-            raise ModuleNotFoundError(
-                f"Module 'shapely' not found. Install optionally with `scale-nucleus[shapely]` or when developing "
-                f"`poetry install -E shapely`. {platform_specific_msg}"
-            )
+    Polygon = PackageNotInstalled
 
 
 from nucleus.annotation import CuboidAnnotation
@@ -375,6 +357,4 @@ def detection_iou(
             meter_3d.append(iou_3d[i, j])
             meter_2d.append(iou_2d[i, j])
 
-    meter_3d = np.array(meter_3d)
-    meter_2d = np.array(meter_2d)
-    return meter_3d, meter_2d
+    return np.array(meter_3d), np.array(meter_2d)
