@@ -2,7 +2,7 @@ from typing import Dict, List, Optional, Union
 
 import requests
 
-from .constants import METADATA_KEY, NAME_KEY, REFERENCE_ID_KEY
+from .constants import METADATA_KEY, MODEL_TAGS_KEY, NAME_KEY, REFERENCE_ID_KEY
 from .dataset import Dataset
 from .job import AsyncJob
 from .model_run import ModelRun
@@ -77,7 +77,7 @@ class Model:
 
         # For large ingestions, we recommend asynchronous ingestion
         job = dataset.upload_predictions(
-            [prediction_1, prediction_2], asynchronous=True
+            model, [prediction_1, prediction_2], asynchronous=True
         )
         # Check current status
         job.status()
@@ -92,15 +92,26 @@ class Model:
     endpoint, using :meth:`NucleusClient.add_model`.
     """
 
-    def __init__(self, model_id, name, reference_id, metadata, client):
+    def __init__(
+        self,
+        model_id,
+        name,
+        reference_id,
+        metadata,
+        client,
+        bundle_name=None,
+        tags: List[str] = None,
+    ):
         self.id = model_id
         self.name = name
         self.reference_id = reference_id
         self.metadata = metadata
+        self.bundle_name = bundle_name
+        self.tags = tags if tags else []
         self._client = client
 
     def __repr__(self):
-        return f"Model(model_id='{self.id}', name='{self.name}', reference_id='{self.reference_id}', metadata={self.metadata}, client={self._client})"
+        return f"Model(model_id='{self.id}', name='{self.name}', reference_id='{self.reference_id}', metadata={self.metadata}, bundle_name={self.bundle_name}, client={self._client})"
 
     def __eq__(self, other):
         return (
@@ -108,6 +119,7 @@ class Model:
             and (self.name == other.name)
             and (self.metadata == other.metadata)
             and (self._client == other._client)
+            and (self.bundle_name == other.bundle_name)
         )
 
     def __hash__(self):
@@ -187,3 +199,71 @@ class Model:
             requests_command=requests.post,
         )
         return AsyncJob.from_json(response, self._client)
+
+    def run(self, dataset_id: str, slice_id: Optional[str]) -> str:
+        """Runs inference on the bundle associated with the model on the dataset. ::
+
+            import nucleus
+            client = nucleus.NucleusClient("YOUR_SCALE_API_KEY")
+            model = client.list_models()[0]
+
+            model.run("ds_123456")
+
+        Args:
+            dataset_id: id of dataset to run inference on
+            job_id: nucleus job used to track async job progress
+            slice_id: (optional) id of slice of the dataset to run inference on
+        """
+        response = self._client.make_request(
+            {"dataset_id": dataset_id, "slice_id": slice_id},
+            f"model/run/{self.id}/",
+            requests_command=requests.post,
+        )
+
+        return response
+
+    def add_tags(self, tags: List[str]):
+        """Tag the model with custom tag names. ::
+
+            import nucleus
+            client = nucleus.NucleusClient("YOUR_SCALE_API_KEY")
+            model = client.list_models()[0]
+
+            model.add_tags(["tag_A", "tag_B"])
+
+        Args:
+            tags: list of tag names
+        """
+        response = self._client.make_request(
+            {MODEL_TAGS_KEY: tags},
+            f"model/{self.id}/tag",
+            requests_command=requests.post,
+        )
+
+        if response.get("msg", False):
+            self.tags.extend(tags)
+
+        return response
+
+    def remove_tags(self, tags: List[str]):
+        """Remove tag(s) from the model. ::
+
+            import nucleus
+            client = nucleus.NucleusClient("YOUR_SCALE_API_KEY")
+            model = client.list_models()[0]
+
+            model.remove_tags(["tag_x"])
+
+        Args:
+            tags: list of tag names to remove
+        """
+        response = self._client.make_request(
+            {MODEL_TAGS_KEY: tags},
+            f"model/{self.id}/tag",
+            requests_command=requests.delete,
+        )
+
+        if response.get("msg", False):
+            self.tags = list(filter(lambda t: t not in tags, self.tags))
+
+        return response
