@@ -15,7 +15,9 @@ from .constants import (
     ITEMS_KEY,
     LABELS_KEY,
     METADATA_KEY,
+    MODEL_BUNDLE_NAME_KEY,
     MODEL_ID_KEY,
+    MODEL_TAGS_KEY,
     NAME_KEY,
     REFERENCE_ID_KEY,
     SCENES_KEY,
@@ -122,14 +124,25 @@ def construct_box_predictions_payload(
 
 
 def construct_model_creation_payload(
-    name: str, reference_id: str, metadata: Optional[Dict]
+    name: str,
+    reference_id: str,
+    metadata: Optional[Dict],
+    bundle_name: Optional[str],
+    tags: Optional[List[str]],
 ) -> dict:
-    return {
+    payload = {
         NAME_KEY: name,
         REFERENCE_ID_KEY: reference_id,
         METADATA_KEY: metadata if metadata else {},
     }
 
+    if bundle_name:
+        payload[MODEL_BUNDLE_NAME_KEY] = bundle_name
+    if tags:
+        payload[MODEL_TAGS_KEY] = tags
+
+    return payload
+
 
 def construct_model_run_creation_payload(
     name: str,
